/*******************************************************************************

     TEMPORIZADOR P/ FERRO DE SOLDAR

     C/ PIC 16F628A


     Autor: Cludio Lrios

     Data: 16/06/2017

     Aplicao: Desliga o ferro de solda aps tempo definido no programa.
     Opcionalmente poder ser usado para desligar ferro de passar roupa.
     Tempo pode ser ajustado por meio de um interruptor entre 1 a 99 minutos.
     Aviso de um bip curto quando a contagem chegar a 1 minuto.
     Ao desligar emite 5 bips.
     Ajuste do tempo:
     Pressione o interruptor sw_adj. Comear a incrementar. Solte quando estiver
     na regulagem correta. Poder inverter a contagem por soltar o boto e 
     aguardar meio segundo antes de apertar de novo.
     Aps um tempo sem pressionar, o valor mostrado ser salvo na EEprom interna
     do PIC e ser usado sempre como o valor desejado de temporizao.
     
     Uso: O suporte do ferro dever ter uma microchave que desligue com o pso 
     e religue quando retiramos o ferro.
     Toda vez que colocamos o ferro no suporte comea a contar o tempo ajustado.
     Se o ferro for retirado, o tempo ajustado  carregado novamente. Desta forma,
     trabalhando com o ferro, ele se manter ligado. Se deixarmos no suporte, 
     aps o tempo ajustado ele se desligar automticamente.

     Uso didtico

     Este arquivo  parte integrante do blog Lrios.Tecnologia.ws

*******************************************************************************/
// Opo do usario:
/******************************************************************************/

//       #define ANODO_COMUM     //descomente para nodo comum e comente para 
                                 //ctodo comum

//==============================================================================
         #define AUTO_DESLIGAMENTO_TOTAL //depois do bip final desliga tudo
         
//Dever ser acrescentado um boto de Start paralelo com o rel. A alimentao
//do transformador de fora dever estar em srie com o rele.
//Quando finaliza o tempo, desliga o rel e tambm toda a alimentao do equipa-
//mento. Para religar, retire o ferro do suporte e aperte o boto 'Start' para
// iniciar o ciclo completo.         
//==============================================================================

#include <16F628A.h>
#use delay(clock=4000000)
#fuses NOWDT,HS, PUT, NOPROTECT, BROWNOUT, NOMCLR, NOLVP, NOCPD
#byte portb = 0x06
#byte porta = 0x05
#byte trisa = 0x85
#byte trisb = 0x86
#rom  0x2100 = {60}  //define tempo ps compilao
//BIT

#bit  sw_delay_off = 0x5.5  //interrptor de finalizao do tempo antecipado (na)
#bit  sw_suporte   = 0x5.4  //interruptor colocado no suporte do ferro (nc)
#bit  sw_adj       = 0x6.7  //interruptor de ajuste do tempo de desligamento
#bit  beep         = 0x5.0  //sada do beep (disco piezoeltrico)
#bit  rele         = 0x5.1  //sada para acionar rele
#bit  ak1          = 0x5.3  //sada para multiplexao do display 1
#bit  ak0          = 0x5.2  //sada para multiplexao do display 0



//constantes para converso 7 seguimentos

//                                  PGFEDCBA   SEGUIMENTOS
//                            PORTB 76543210
unsigned int8 const NUM[] = {     0B00111111,//0
                                  0B00000110,//1
                                  0B01011011,//2
                                  0B01001111,//3
                                  0B01100110,//4
                                  0B01101101,//5
                                  0B01111101,//6
                                  0B00000111,//7
                                  0B01111111,//8
                                  0B01101111 //9
                                                   };

 //defines
 #define TEMPO_OUT_PROG  12 // tempo de time_out
 #define TEMPO_INV       7 // tempo para inverter entre incremento ou decremento
 #define VELOCIDADE_INC  400 // 400 milissegundos
 #define TEMPO_MAX   99      // 99  minutos


 //variveis globais
 unsigned int8 tempo=TEMPO_OUT_PROG; //tempo de time_out do modo de programao
 unsigned int8 minutos,segundos; //contadores de minutos e segundos
 unsigned int8 vel=VELOCIDADE_INC;
 unsigned int1 f_bipar=0; //indica para produzir um bip na sada do piezo
 unsigned int1 f_prog=0;  //modo de programao ativo
 unsigned int1 f_inv=0; //inverte funo entre incremento e decremento
 unsigned int1 f_inv_old=0;//auxiliar
 unsigned int1 f_t1=0; //auxiliar
 unsigned int1 f_bipar_c=0;//auxiliar para beep curto
  /******************************************************************************
           MULTIPLEXA OS DISPLAYS
******************************************************************************/

 void multiplexa(){
    static int8 vez;
 
   if(++vez>1) vez=0;
    porta&=0xf3;//apaga display (resetando)
    #ifdef ANODO_COMUM
    portb=0xff;
    #else
    portb=0;
    #endif

    switch (vez){

     case 0: {
       portb=num[minutos/10];
       #ifdef ANODO_COMUM
       portb=~portb;
       #endif
       porta= porta|8; break;}
     case 1: {
       portb=num[minutos%10];


       #ifdef ANODO_COMUM
       portb=~portb;
       #endif
       porta= porta|4; break;}
    }
 }

/******************************************************************************
           ROTINA PARA PRODUO DE 'BEEP' CURTO
******************************************************************************/

void bip(){
 unsigned int16 c;
     ak1=0;//apaga displays
     ak0=0;
    for(c=0; c<2500; c++) {
      beep=1;
      delay_us(72); //ajuste conforme freq. ressonancia do piezoeletrico
      beep=0;
      delay_us(132);
    }    
}

/******************************************************************************
           ROTINA PARA PRODUO DE 'BEEP' COMPOSTO
******************************************************************************/


//rotina de produo de som no disco piezoeletrico
 void bipar() {
   unsigned int8 a;
   
   if(f_bipar_c){
     disable_interrupts(int_timer0);
     bip();
     bip();
     enable_interrupts(int_timer0);
     f_bipar_c=0;//aviso de 1 minuto 
   }
         
   if(!f_bipar)return;

   for(a=0;a<5;a++){
    bip();
    delay_ms(100);//delay entre beeps
  }
   f_bipar=0;
    #ifdef AUTO_DESLIGAMENTO_TOTAL
     rele=0;
    #endif
   
 }

/******************************************************************************
           ROTINA DE TESTE DO INTERRUPTOR DE AJUSTE DE TEMPO
******************************************************************************/
void testa_adj(){
    if(vel)vel--;
    if(vel)return;
    vel=VELOCIDADE_INC;

    if(!sw_adj){
    if(!f_prog)minutos=read_eeprom(0); //busca valor atual de regulagem de tempo
    f_prog=1; //entra em modo de ajuste do tempo
    f_t1=0;
    
    if(f_inv){
      if(minutos<TEMPO_MAX)minutos++;
    }
    else{ if(minutos)minutos--;}


    }
    else{
     if(f_prog){
        if(tempo) tempo--;
        if((tempo<=TEMPO_INV)&& !f_t1){f_inv=!f_inv; f_t1=1;}
        if(!tempo){
           tempo=TEMPO_OUT_PROG;
           write_eeprom(0,minutos);//salva ajuste
           f_prog=0; //sai do modo de ajuste de tempo
           ak1=0;
           ak0=0;
           delay_ms(500);
           rele=1;//liga sada ao sair do modo de ajuste
       }
     }
    }


    if(f_inv!=f_inv_old){tempo=TEMPO_OUT_PROG; f_inv_old=f_inv;}//reinicia tempo


}
/******************************************************************************
       ROTINA DE TESTE DOS INTERRUPTORES 'SUPORTE' E 'DELAY_OFF'
******************************************************************************/
 void testa_sw(){
     if(f_prog)return;
     if(sw_suporte){//se ferro de soldar est no suporte
       if(rele)enable_interrupts(int_timer0);//conta tempo

     }
     else{//ferro de soldar fora do suporte
      disable_interrupts(int_timer0);//para contagem de tempo
      segundos=59;//reinicia tempo
      minutos=read_eeprom(0);
      rele=1;

     }

    if(sw_suporte && !sw_delay_off){// testa se foi apertado 'delay_off'
      disable_interrupts(int_timer0);
      rele=0;
      minutos=0;
      segundos=59;//reinicia tempo
    }

 }

/******************************************************************************
           ROTINA DE INTERRUPO DO TIMER 0
******************************************************************************/
#INT_TIMER0
void INT_TIMER0_ISR(){
 static int32 cnt32;
//--------------------------------------------------------
//gera 1 segundos com metodo de acumulo de erro e pode ser usado
// para cristais com valores quebrados das mais variadas frequencias
// ajuste o timer para o prescaller igual abaixo no main

 #define presc  64 // valor do prescaller do timer usado (8bits)
 #define fator  (256*presc*4)
 #define f_xtal 4000000  //frequencia do cristal usado (mesmo do cabealho)

   cnt32+=fator;
   if(cnt32>=f_xtal){
     cnt32-=f_xtal;

      if(--segundos==0xff){
        segundos=59;
        if( minutos>0) --minutos;
           }
     if((minutos==1)&&(segundos==59))f_bipar_c=1;
          
           
     if(!minutos) {
       #ifndef AUTO_DESLIGAMENTO_TOTAL
        rele=0;//desliga o rele de sada
       #endif
        disable_interrupts(int_timer0);//para a contagem
        f_bipar=1;

     }
   }
 }
/*******************************************************************************
          Rotina Main
*******************************************************************************/
void main() {

   setup_counters(RTCC_INTERNAL,RTCC_DIV_64);//timer 0 com prescaller /64
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_comparator(NC_NC_NC_NC);//desliga comparadores analgicos
   setup_vref(FALSE);
   port_b_pullups(true);//liga pullup do pino 13
   trisb=0X80;//sentido do portB
   trisa=0b11110000;//sentido do portA
   minutos=read_eeprom(0);//recupera tempo da e
   segundos=0;
   enable_interrupts(global);
/*******************************************************************************
      Loop  de Repetio
*******************************************************************************/

   for(;;){
     multiplexa();//multiplexa displays
     testa_adj();//testa interruptor de ajuste de tempo
     testa_sw();//teta interruptores
     bipar();//testa se deve bipar
     delay_ms(1);//delay de 1 ms
   }//for(;;)

}//main()
//------------------------FIM DO PROGRAMA---------------------------------------
